
<?php  class AopClient 
{
	public $appId = NULL;
	public $rsaPrivateKeyFilePath = NULL;
	public $rsaPrivateKey = NULL;
	public $gatewayUrl = "https://openapi.alipay.com/gateway.do";
	public $format = "json";
	public $apiVersion = "1.0";
	public $postCharset = "UTF-8";
	public $alipayPublicKey = NULL;
	public $alipayrsaPublicKey = NULL;
	public $debugInfo = false;
	private $fileCharset = "UTF-8";
	private $RESPONSE_SUFFIX = "_response";
	private $ERROR_RESPONSE = "error_response";
	private $SIGN_NODE_NAME = "sign";
	private $ENCRYPT_XML_NODE_NAME = "response_encrypted";
	private $needEncrypt = false;
	public $signType = "RSA";
	public $encryptKey = NULL;
	public $encryptType = "AES";
	protected $alipaySdkVersion = "alipay-sdk-php-20160411";
	public function generateSign($params, $signType = "RSA") 
	{
		return $this->sign($this->getSignContent($params), $signType);
	}
	public function rsaSign($params, $signType = "RSA") 
	{
		return $this->sign($this->getSignContent($params), $signType);
	}
	protected function getSignContent($params) 
	{
		ksort($params);
		$stringToBeSigned = "";
		$i = 0;
		foreach( $params as $k => $v ) 
		{
			if( false === $this->checkEmpty($v) && "@" != substr($v, 0, 1) ) 
			{
				$v = $this->characet($v, $this->postCharset);
				if( $i == 0 ) 
				{
					$stringToBeSigned .= (string) $k . "=" . (string) $v;
				}
				else 
				{
					$stringToBeSigned .= "&" . (string) $k . "=" . (string) $v;
				}
				$i++;
			}
		}
		unset($k);
		unset($v);
		return $stringToBeSigned;
	}
	protected function sign($data, $signType = "RSA") 
	{
		if( $this->checkEmpty($this->rsaPrivateKeyFilePath) ) 
		{
			$priKey = $this->rsaPrivateKey;
			$res = "-----BEGIN RSA PRIVATE KEY-----\n" . wordwrap($priKey, 64, "\n", true) . "\n-----END RSA PRIVATE KEY-----";
		}
		else 
		{
			$priKey = file_get_contents($this->rsaPrivateKeyFilePath);
			$res = openssl_get_privatekey($priKey);
		}
		$res or exit( "您使用的私钥格式错误，请检查RSA私钥配置" );
		if( "RSA2" == $signType ) 
		{
			openssl_sign($data, $sign, $res, OPENSSL_ALGO_SHA256);
		}
		else 
		{
			openssl_sign($data, $sign, $res);
		}
		if( !$this->checkEmpty($this->rsaPrivateKeyFilePath) ) 
		{
			openssl_free_key($res);
		}
		$sign = base64_encode($sign);
		return $sign;
	}
	protected function curl($url, $postFields = NULL) 
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_FAILONERROR, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		$postBodyString = "";
		$encodeArray = array( );
		$postMultipart = false;
		if( is_array($postFields) && 0 < count($postFields) ) 
		{
			foreach( $postFields as $k => $v ) 
			{
				if( "@" != substr($v, 0, 1) ) 
				{
					$postBodyString .= (string) $k . "=" . urlencode($this->characet($v, $this->postCharset)) . "&";
					$encodeArray[$k] = $this->characet($v, $this->postCharset);
				}
				else 
				{
					$postMultipart = true;
					$encodeArray[$k] = new CURLFile(substr($v, 1));
				}
			}
			unset($k);
			unset($v);
			curl_setopt($ch, CURLOPT_POST, true);
			if( $postMultipart ) 
			{
				curl_setopt($ch, CURLOPT_POSTFIELDS, $encodeArray);
			}
			else 
			{
				curl_setopt($ch, CURLOPT_POSTFIELDS, substr($postBodyString, 0, -1));
			}
		}
		if( $postMultipart ) 
		{
			$headers = array( "content-type: multipart/form-data;charset=" . $this->postCharset . ";boundary=" . $this->getMillisecond() );
		}
		else 
		{
			$headers = array( "content-type: application/x-www-form-urlencoded;charset=" . $this->postCharset );
		}
		curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		$reponse = curl_exec($ch);
		if( curl_errno($ch) ) 
		{
			throw new Exception(curl_error($ch), 0);
		}
		$httpStatusCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		if( 200 !== $httpStatusCode ) 
		{
			throw new Exception($reponse, $httpStatusCode);
		}
		curl_close($ch);
		return $reponse;
	}
	protected function getMillisecond() 
	{
		list($s1, $s2) = explode(" ", microtime());
		return (double) sprintf("%.0f", (floatval($s1) + floatval($s2)) * 1000);
	}
	public function pageExecute($request, $httpmethod = "POST") 
	{
		if( $this->checkEmpty($this->postCharset) ) 
		{
			$this->postCharset = "UTF-8";
		}
		$this->fileCharset = mb_detect_encoding($this->appId, "UTF-8,GBK");
		if( strcasecmp($this->fileCharset, $this->postCharset) ) 
		{
			throw new Exception("文件编码：[" . $this->fileCharset . "] 与表单提交编码：[" . $this->postCharset . "]两者不一致!");
		}
		$iv = NULL;
		if( !$this->checkEmpty($request->getApiVersion()) ) 
		{
			$iv = $request->getApiVersion();
		}
		else 
		{
			$iv = $this->apiVersion;
		}
		$sysParams["app_id"] = $this->appId;
		$sysParams["version"] = $iv;
		$sysParams["format"] = $this->format;
		$sysParams["sign_type"] = $this->signType;
		$sysParams["method"] = $request->getApiMethodName();
		$sysParams["timestamp"] = date("Y-m-d H:i:s");
		$sysParams["alipay_sdk"] = $this->alipaySdkVersion;
		$sysParams["terminal_type"] = $request->getTerminalType();
		$sysParams["terminal_info"] = $request->getTerminalInfo();
		$sysParams["prod_code"] = $request->getProdCode();
		$sysParams["notify_url"] = $request->getNotifyUrl();
		$sysParams["return_url"] = $request->getReturnUrl();
		$sysParams["charset"] = $this->postCharset;
		$apiParams = $request->getApiParas();
		if( method_exists($request, "getNeedEncrypt") && $request->getNeedEncrypt() ) 
		{
			$sysParams["encrypt_type"] = $this->encryptType;
			if( $this->checkEmpty($apiParams["biz_content"]) ) 
			{
				throw new Exception(" api request Fail! The reason : encrypt request is not supperted!");
			}
			if( $this->checkEmpty($this->encryptKey) || $this->checkEmpty($this->encryptType) ) 
			{
				throw new Exception(" encryptType and encryptKey must not null! ");
			}
			if( "AES" != $this->encryptType ) 
			{
				throw new Exception("加密类型只支持AES");
			}
			$enCryptContent = encrypt($apiParams["biz_content"], $this->encryptKey);
			$apiParams["biz_content"] = $enCryptContent;
		}
		$totalParams = array_merge($apiParams, $sysParams);
		$preSignStr = $this->getSignContent($totalParams);
		$totalParams["sign"] = $this->generateSign($totalParams, $this->signType);
		if( "GET" == $httpmethod ) 
		{
			$requestUrl = $this->gatewayUrl . "?" . $preSignStr . "&sign=" . urlencode($totalParams["sign"]);
			return $requestUrl;
		}
		return $this->buildRequestForm($totalParams);
	}
	protected function buildRequestForm($para_temp) 
	{
		$sHtml = "<form id='alipaysubmit' name='alipaysubmit' action='" . $this->gatewayUrl . "?charset=" . trim($this->postCharset) . "' method='POST'>";
		while( list($key, $val) = each($para_temp) ) 
		{
			if( false === $this->checkEmpty($val) ) 
			{
				$val = str_replace("'", "&apos;", $val);
				$sHtml .= "<input type='hidden' name='" . $key . "' value='" . $val . "'/>";
			}
		}
		$sHtml = $sHtml . "<input type='submit' value='ok' style='display:none;''></form>";
		$sHtml = $sHtml . "<script>document.forms['alipaysubmit'].submit();</script>";
		return $sHtml;
	}
	public function execute($request, $authToken = NULL, $appInfoAuthtoken = NULL) 
	{
		if( $this->checkEmpty($this->postCharset) ) 
		{
			$this->postCharset = "UTF-8";
		}
		$this->fileCharset = mb_detect_encoding($this->appId, "UTF-8,GBK");
		if( strcasecmp($this->fileCharset, $this->postCharset) ) 
		{
			throw new Exception("文件编码：[" . $this->fileCharset . "] 与表单提交编码：[" . $this->postCharset . "]两者不一致!");
		}
		$iv = NULL;
		if( !$this->checkEmpty($request->getApiVersion()) ) 
		{
			$iv = $request->getApiVersion();
		}
		else 
		{
			$iv = $this->apiVersion;
		}
		$sysParams["app_id"] = $this->appId;
		$sysParams["version"] = $iv;
		$sysParams["format"] = $this->format;
		$sysParams["sign_type"] = $this->signType;
		$sysParams["method"] = $request->getApiMethodName();
		$sysParams["timestamp"] = date("Y-m-d H:i:s");
		$sysParams["auth_token"] = $authToken;
		$sysParams["alipay_sdk"] = $this->alipaySdkVersion;
		$sysParams["terminal_type"] = $request->getTerminalType();
		$sysParams["terminal_info"] = $request->getTerminalInfo();
		$sysParams["prod_code"] = $request->getProdCode();
		$sysParams["notify_url"] = $request->getNotifyUrl();
		$sysParams["charset"] = $this->postCharset;
		$sysParams["app_auth_token"] = $appInfoAuthtoken;
		$apiParams = $request->getApiParas();
		if( method_exists($request, "getNeedEncrypt") && $request->getNeedEncrypt() ) 
		{
			$sysParams["encrypt_type"] = $this->encryptType;
			if( $this->checkEmpty($apiParams["biz_content"]) ) 
			{
				throw new Exception(" api request Fail! The reason : encrypt request is not supperted!");
			}
			if( $this->checkEmpty($this->encryptKey) || $this->checkEmpty($this->encryptType) ) 
			{
				throw new Exception(" encryptType and encryptKey must not null! ");
			}
			if( "AES" != $this->encryptType ) 
			{
				throw new Exception("加密类型只支持AES");
			}
			$enCryptContent = encrypt($apiParams["biz_content"], $this->encryptKey);
			$apiParams["biz_content"] = $enCryptContent;
		}
		$sysParams["sign"] = $this->generateSign(array_merge($apiParams, $sysParams), $this->signType);
		$requestUrl = $this->gatewayUrl . "?";
		foreach( $sysParams as $sysParamKey => $sysParamValue ) 
		{
			$requestUrl .= (string) $sysParamKey . "=" . urlencode($this->characet($sysParamValue, $this->postCharset)) . "&";
		}
		$requestUrl = substr($requestUrl, 0, -1);
		try 
		{
			$resp = $this->curl($requestUrl, $apiParams);
		}
		catch( Exception $e ) 
		{
			return false;
		}
		$respWellFormed = false;
		$r = iconv($this->postCharset, $this->fileCharset . "//IGNORE", $resp);
		$signData = NULL;
		if( "json" == $this->format ) 
		{
			$respObject = json_decode($r);
			if( NULL !== $respObject ) 
			{
				$respWellFormed = true;
				$signData = $this->parserJSONSignData($request, $resp, $respObject);
			}
		}
		else 
		{
			if( "xml" == $this->format ) 
			{
				$respObject = @simplexml_load_string($resp);
				if( false !== $respObject ) 
				{
					$respWellFormed = true;
					$signData = $this->parserXMLSignData($request, $resp);
				}
			}
		}
		if( false === $respWellFormed ) 
		{
			return false;
		}
		$this->checkResponseSign($request, $signData, $resp, $respObject);
		if( method_exists($request, "getNeedEncrypt") && $request->getNeedEncrypt() ) 
		{
			if( "json" == $this->format ) 
			{
				$resp = $this->encryptJSONSignSource($request, $resp);
				$r = iconv($this->postCharset, $this->fileCharset . "//IGNORE", $resp);
				$respObject = json_decode($r);
			}
			else 
			{
				$resp = $this->encryptXMLSignSource($request, $resp);
				$r = iconv($this->postCharset, $this->fileCharset . "//IGNORE", $resp);
				$respObject = @simplexml_load_string($r);
			}
		}
		return $respObject;
	}
	public function characet($data, $targetCharset) 
	{
		if( !empty($data) ) 
		{
			$fileType = $this->fileCharset;
			if( strcasecmp($fileType, $targetCharset) != 0 ) 
			{
				$data = mb_convert_encoding($data, $targetCharset);
			}
		}
		return $data;
	}
	public function exec($paramsArray) 
	{
		if( !isset($paramsArray["method"]) ) 
		{
			trigger_error("No api name passed");
		}
		$inflector = new LtInflector();
		$inflector->conf["separator"] = ".";
		$requestClassName = ucfirst($inflector->camelize(substr($paramsArray["method"], 7))) . "Request";
		if( !class_exists($requestClassName) ) 
		{
			trigger_error("No such api: " . $paramsArray["method"]);
		}
		$session = (isset($paramsArray["session"]) ? $paramsArray["session"] : NULL);
		$req = new $requestClassName();
		foreach( $paramsArray as $paraKey => $paraValue ) 
		{
			$inflector->conf["separator"] = "_";
			$setterMethodName = $inflector->camelize($paraKey);
			$inflector->conf["separator"] = ".";
			$setterMethodName = "set" . $inflector->camelize($setterMethodName);
			if( method_exists($req, $setterMethodName) ) 
			{
				$req->$setterMethodName($paraValue);
			}
		}
		return $this->execute($req, $session);
	}
	protected function checkEmpty($value) 
	{
		if( !isset($value) ) 
		{
			return true;
		}
		if( $value === NULL ) 
		{
			return true;
		}
		if( trim($value) === "" ) 
		{
			return true;
		}
		return false;
	}
	public function rsaCheckV1($params, $rsaPublicKeyFilePath) 
	{
		$sign = $params["sign"];
		$params["sign_type"] = NULL;
		$params["sign"] = NULL;
		return $this->verify($this->getSignContent($params), $sign, $rsaPublicKeyFilePath);
	}
	public function rsaCheckV2($params, $rsaPublicKeyFilePath) 
	{
		$sign = $params["sign"];
		$params["sign"] = NULL;
		return $this->verify($this->getSignContent($params), $sign, $rsaPublicKeyFilePath);
	}
	public function verify($data, $sign, $rsaPublicKeyFilePath, $signType = "RSA") 
	{
		if( $this->checkEmpty($this->alipayPublicKey) ) 
		{
			$pubKey = $this->alipayrsaPublicKey;
			$res = "-----BEGIN PUBLIC KEY-----\n" . wordwrap($pubKey, 64, "\n", true) . "\n-----END PUBLIC KEY-----";
		}
		else 
		{
			$pubKey = file_get_contents($rsaPublicKeyFilePath);
			$res = openssl_get_publickey($pubKey);
		}
		$res or exit( "支付宝RSA公钥错误。请检查公钥文件格式是否正确" );
		if( "RSA2" == $signType ) 
		{
			$result = (bool) openssl_verify($data, base64_decode($sign), $res, OPENSSL_ALGO_SHA256);
		}
		else 
		{
			$result = (bool) openssl_verify($data, base64_decode($sign), $res);
		}
		if( !$this->checkEmpty($this->alipayPublicKey) ) 
		{
			openssl_free_key($res);
		}
		return $result;
	}
	public function checkSignAndDecrypt($params, $rsaPublicKeyPem, $rsaPrivateKeyPem, $isCheckSign, $isDecrypt) 
	{
		$charset = $params["charset"];
		$bizContent = $params["biz_content"];
		if( $isCheckSign && !$this->rsaCheckV2($params, $rsaPublicKeyPem) ) 
		{
			echo "<br/>checkSign failure<br/>";
			exit();
		}
		if( $isDecrypt ) 
		{
			return $this->rsaDecrypt($bizContent, $rsaPrivateKeyPem, $charset);
		}
		return $bizContent;
	}
	public function encryptAndSign($bizContent, $rsaPublicKeyPem, $rsaPrivateKeyPem, $charset, $isEncrypt, $isSign) 
	{
		if( $isEncrypt && $isSign ) 
		{
			$encrypted = $this->rsaEncrypt($bizContent, $rsaPublicKeyPem, $charset);
			$sign = $this->sign($bizContent);
			$response = "<?xml version=\"1.0\" encoding=\"" . $charset . "\"?><alipay><response>" . $encrypted . "</response><encryption_type>RSA</encryption_type><sign>" . $sign . "</sign><sign_type>RSA</sign_type></alipay>";
			return $response;
		}
		if( $isEncrypt && !$isSign ) 
		{
			$encrypted = $this->rsaEncrypt($bizContent, $rsaPublicKeyPem, $charset);
			$response = "<?xml version=\"1.0\" encoding=\"" . $charset . "\"?><alipay><response>" . $encrypted . "</response><encryption_type>RSA</encryption_type></alipay>";
			return $response;
		}
		if( !$isEncrypt && $isSign ) 
		{
			$sign = $this->sign($bizContent);
			$response = "<?xml version=\"1.0\" encoding=\"" . $charset . "\"?><alipay><response>" . $bizContent . "</response><sign>" . $sign . "</sign><sign_type>RSA</sign_type></alipay>";
			return $response;
		}
		$response = "<?xml version=\"1.0\" encoding=\"" . $charset . "\"?>" . $bizContent;
		return $response;
	}
	public function rsaEncrypt($data, $rsaPublicKeyPem, $charset) 
	{
		$pubKey = file_get_contents($rsaPublicKeyPem);
		$res = openssl_get_publickey($pubKey);
		$blocks = $this->splitCN($data, 0, 30, $charset);
		$_obfuscated_63687274657874C2A0_ = NULL;
		$_obfuscated_656E636F646573C2A0_ = array( );
		foreach( $blocks as $n => $block ) 
		{
			if( !openssl_public_encrypt($block, $_obfuscated_63687274657874C2A0_, $res) ) 
			{
				echo "<br/>" . openssl_error_string() . "<br/>";
			}
			$encodes[] = $_obfuscated_63687274657874C2A0_;
		}
		$chrtext = implode(",", $encodes);
		return $chrtext;
	}
	public function rsaDecrypt($data, $rsaPrivateKeyPem, $charset) 
	{
		$priKey = file_get_contents($rsaPrivateKeyPem);
		$res = openssl_get_privatekey($priKey);
		$decodes = explode(",", $data);
		$strnull = "";
		$dcyCont = "";
		foreach( $decodes as $n => $decode ) 
		{
			if( !openssl_private_decrypt($decode, $dcyCont, $res) ) 
			{
				echo "<br/>" . openssl_error_string() . "<br/>";
			}
			$strnull .= $dcyCont;
		}
		return $strnull;
	}
	public function splitCN($cont, $n = 0, $subnum, $charset) 
	{
		$arrr = array( );
		$i = $n;
		while( $i < strlen($cont) ) 
		{
			$res = $this->subCNchar($cont, $i, $subnum, $charset);
			if( !empty($res) ) 
			{
				$arrr[] = $res;
			}
			$i += $subnum;
		}
		return $arrr;
	}
	public function subCNchar($str, $start = 0, $length, $charset = "gbk") 
	{
		if( strlen($str) <= $length ) 
		{
			return $str;
		}
		$re["utf-8"] = "/[\x01-]|[?-?][?-?]|[?-?][?-?]{2}|[?-?][?-?]{3}/";
		$re["gb2312"] = "/[\x01-]|[?-?][?-?]/";
		$re["gbk"] = "/[\x01-]|[?-?][@-?]/";
		$re["big5"] = "/[\x01-]|[?-?]([@-~]|?-?])/";
		preg_match_all($re[$charset], $str, $match);
		$slice = join("", array_slice($match[0], $start, $length));
		return $slice;
	}
	public function parserResponseSubCode($request, $responseContent, $respObject, $format) 
	{
		if( "json" == $format ) 
		{
			$apiName = $request->getApiMethodName();
			$rootNodeName = str_replace(".", "_", $apiName) . $this->RESPONSE_SUFFIX;
			$errorNodeName = $this->ERROR_RESPONSE;
			$rootIndex = strpos($responseContent, $rootNodeName);
			$errorIndex = strpos($responseContent, $errorNodeName);
			if( 0 < $rootIndex ) 
			{
				$rInnerObject = $respObject->$rootNodeName;
			}
			else 
			{
				if( 0 < $errorIndex ) 
				{
					$rInnerObject = $respObject->$errorNodeName;
				}
				else 
				{
					return NULL;
				}
			}
			if( isset($rInnerObject->sub_code) ) 
			{
				return $rInnerObject->sub_code;
			}
			return NULL;
		}
		if( "xml" == $format ) 
		{
			return $respObject->sub_code;
		}
	}
	public function parserJSONSignData($request, $responseContent, $responseJSON) 
	{
		$signData = new SignData();
		$signData->sign = $this->parserJSONSign($responseJSON);
		$signData->signSourceData = $this->parserJSONSignSource($request, $responseContent);
		return $signData;
	}
	public function parserJSONSignSource($request, $responseContent) 
	{
		$apiName = $request->getApiMethodName();
		$rootNodeName = str_replace(".", "_", $apiName) . $this->RESPONSE_SUFFIX;
		$rootIndex = strpos($responseContent, $rootNodeName);
		$errorIndex = strpos($responseContent, $this->ERROR_RESPONSE);
		if( 0 < $rootIndex ) 
		{
			return $this->parserJSONSource($responseContent, $rootNodeName, $rootIndex);
		}
		if( 0 < $errorIndex ) 
		{
			return $this->parserJSONSource($responseContent, $this->ERROR_RESPONSE, $errorIndex);
		}
	}
	public function parserJSONSource($responseContent, $nodeName, $nodeIndex) 
	{
		$signDataStartIndex = $nodeIndex + strlen($nodeName) + 2;
		$signIndex = strpos($responseContent, "\"" . $this->SIGN_NODE_NAME . "\"");
		$signDataEndIndex = $signIndex - 1;
		$indexLen = $signDataEndIndex - $signDataStartIndex;
		if( $indexLen < 0 ) 
		{
			return NULL;
		}
		return substr($responseContent, $signDataStartIndex, $indexLen);
	}
	public function parserJSONSign($responseJSon) 
	{
		return $responseJSon->sign;
	}
	public function parserXMLSignData($request, $responseContent) 
	{
		$signData = new SignData();
		$signData->sign = $this->parserXMLSign($responseContent);
		$signData->signSourceData = $this->parserXMLSignSource($request, $responseContent);
		return $signData;
	}
	public function parserXMLSignSource($request, $responseContent) 
	{
		$apiName = $request->getApiMethodName();
		$rootNodeName = str_replace(".", "_", $apiName) . $this->RESPONSE_SUFFIX;
		$rootIndex = strpos($responseContent, $rootNodeName);
		$errorIndex = strpos($responseContent, $this->ERROR_RESPONSE);
		if( 0 < $rootIndex ) 
		{
			return $this->parserXMLSource($responseContent, $rootNodeName, $rootIndex);
		}
		if( 0 < $errorIndex ) 
		{
			return $this->parserXMLSource($responseContent, $this->ERROR_RESPONSE, $errorIndex);
		}
	}
	public function parserXMLSource($responseContent, $nodeName, $nodeIndex) 
	{
		$signDataStartIndex = $nodeIndex + strlen($nodeName) + 1;
		$signIndex = strpos($responseContent, "<" . $this->SIGN_NODE_NAME . ">");
		$signDataEndIndex = $signIndex - 1;
		$indexLen = $signDataEndIndex - $signDataStartIndex + 1;
		if( $indexLen < 0 ) 
		{
			return NULL;
		}
		return substr($responseContent, $signDataStartIndex, $indexLen);
	}
	public function parserXMLSign($responseContent) 
	{
		$signNodeName = "<" . $this->SIGN_NODE_NAME . ">";
		$signEndNodeName = "</" . $this->SIGN_NODE_NAME . ">";
		$indexOfSignNode = strpos($responseContent, $signNodeName);
		$indexOfSignEndNode = strpos($responseContent, $signEndNodeName);
		if( $indexOfSignNode < 0 || $indexOfSignEndNode < 0 ) 
		{
			return NULL;
		}
		$nodeIndex = $indexOfSignNode + strlen($signNodeName);
		$indexLen = $indexOfSignEndNode - $nodeIndex;
		if( $indexLen < 0 ) 
		{
			return NULL;
		}
		return substr($responseContent, $nodeIndex, $indexLen);
	}
	public function checkResponseSign($request, $signData, $resp, $respObject) 
	{
		if( !$this->checkEmpty($this->alipayPublicKey) || !$this->checkEmpty($this->alipayrsaPublicKey) ) 
		{
			if( $signData == NULL || $this->checkEmpty($signData->sign) || $this->checkEmpty($signData->signSourceData) ) 
			{
				throw new Exception(" check sign Fail! The reason : signData is Empty");
			}
			$responseSubCode = $this->parserResponseSubCode($request, $resp, $respObject, $this->format);
			if( !$this->checkEmpty($responseSubCode) || $this->checkEmpty($responseSubCode) && !$this->checkEmpty($signData->sign) ) 
			{
				$checkResult = $this->verify($signData->signSourceData, $signData->sign, $this->alipayPublicKey, $this->signType);
				if( !$checkResult ) 
				{
					if( 0 < strpos($signData->signSourceData, "\\/") ) 
					{
						$signData->signSourceData = str_replace("\\/", "/", $signData->signSourceData);
						$checkResult = $this->verify($signData->signSourceData, $signData->sign, $this->alipayPublicKey, $this->signType);
						if( !$checkResult ) 
						{
							throw new Exception("check sign Fail! [sign=" . $signData->sign . ", signSourceData=" . $signData->signSourceData . "]");
						}
					}
					else 
					{
						throw new Exception("check sign Fail! [sign=" . $signData->sign . ", signSourceData=" . $signData->signSourceData . "]");
					}
				}
			}
		}
	}
	private function encryptJSONSignSource($request, $responseContent) 
	{
		$parsetItem = $this->parserEncryptJSONSignSource($request, $responseContent);
		$bodyIndexContent = substr($responseContent, 0, $parsetItem->startIndex);
		$bodyEndContent = substr($responseContent, $parsetItem->endIndex, (strlen($responseContent) + 1) - $parsetItem->endIndex);
		$bizContent = decrypt($parsetItem->encryptContent, $this->encryptKey);
		return $bodyIndexContent . $bizContent . $bodyEndContent;
	}
	private function parserEncryptJSONSignSource($request, $responseContent) 
	{
		$apiName = $request->getApiMethodName();
		$rootNodeName = str_replace(".", "_", $apiName) . $this->RESPONSE_SUFFIX;
		$rootIndex = strpos($responseContent, $rootNodeName);
		$errorIndex = strpos($responseContent, $this->ERROR_RESPONSE);
		if( 0 < $rootIndex ) 
		{
			return $this->parserEncryptJSONItem($responseContent, $rootNodeName, $rootIndex);
		}
		if( 0 < $errorIndex ) 
		{
			return $this->parserEncryptJSONItem($responseContent, $this->ERROR_RESPONSE, $errorIndex);
		}
	}
	private function parserEncryptJSONItem($responseContent, $nodeName, $nodeIndex) 
	{
		$signDataStartIndex = $nodeIndex + strlen($nodeName) + 2;
		$signIndex = strpos($responseContent, "\"" . $this->SIGN_NODE_NAME . "\"");
		$signDataEndIndex = $signIndex - 1;
		if( $signDataEndIndex < 0 ) 
		{
			$signDataEndIndex = strlen($responseContent) - 1;
		}
		$indexLen = $signDataEndIndex - $signDataStartIndex;
		$encContent = substr($responseContent, $signDataStartIndex + 1, $indexLen - 2);
		$encryptParseItem = new EncryptParseItem();
		$encryptParseItem->encryptContent = $encContent;
		$encryptParseItem->startIndex = $signDataStartIndex;
		$encryptParseItem->endIndex = $signDataEndIndex;
		return $encryptParseItem;
	}
	private function encryptXMLSignSource($request, $responseContent) 
	{
		$parsetItem = $this->parserEncryptXMLSignSource($request, $responseContent);
		$bodyIndexContent = substr($responseContent, 0, $parsetItem->startIndex);
		$bodyEndContent = substr($responseContent, $parsetItem->endIndex, (strlen($responseContent) + 1) - $parsetItem->endIndex);
		$bizContent = decrypt($parsetItem->encryptContent, $this->encryptKey);
		return $bodyIndexContent . $bizContent . $bodyEndContent;
	}
	private function parserEncryptXMLSignSource($request, $responseContent) 
	{
		$apiName = $request->getApiMethodName();
		$rootNodeName = str_replace(".", "_", $apiName) . $this->RESPONSE_SUFFIX;
		$rootIndex = strpos($responseContent, $rootNodeName);
		$errorIndex = strpos($responseContent, $this->ERROR_RESPONSE);
		if( 0 < $rootIndex ) 
		{
			return $this->parserEncryptXMLItem($responseContent, $rootNodeName, $rootIndex);
		}
		if( 0 < $errorIndex ) 
		{
			return $this->parserEncryptXMLItem($responseContent, $this->ERROR_RESPONSE, $errorIndex);
		}
	}
	private function parserEncryptXMLItem($responseContent, $nodeName, $nodeIndex) 
	{
		$signDataStartIndex = $nodeIndex + strlen($nodeName) + 1;
		$xmlStartNode = "<" . $this->ENCRYPT_XML_NODE_NAME . ">";
		$xmlEndNode = "</" . $this->ENCRYPT_XML_NODE_NAME . ">";
		$indexOfXmlNode = strpos($responseContent, $xmlEndNode);
		if( $indexOfXmlNode < 0 ) 
		{
			$item = new EncryptParseItem();
			$item->encryptContent = NULL;
			$item->startIndex = 0;
			$item->endIndex = 0;
			return $item;
		}
		$startIndex = $signDataStartIndex + strlen($xmlStartNode);
		$bizContentLen = $indexOfXmlNode - $startIndex;
		$bizContent = substr($responseContent, $startIndex, $bizContentLen);
		$encryptParseItem = new EncryptParseItem();
		$encryptParseItem->encryptContent = $bizContent;
		$encryptParseItem->startIndex = $signDataStartIndex;
		$encryptParseItem->endIndex = $indexOfXmlNode + strlen($xmlEndNode);
		return $encryptParseItem;
	}
	public function echoDebug($content) 
	{
		if( $this->debugInfo ) 
		{
			echo "<br/>" . $content;
		}
	}
}
class AlipayTradeService 
{
	public $gateway_url = "https://openapi.alipay.com/gateway.do";
	public $notify_url = NULL;
	public $alipay_public_key = NULL;
	public $private_key = NULL;
	public $appid = NULL;
	public $charset = "UTF-8";
	public $token = NULL;
	private $MaxQueryRetry = NULL;
	private $QueryDuration = NULL;
	public $format = "json";
	public function __construct($alipay_config) 
	{
		$this->gateway_url = $alipay_config["gatewayUrl"];
		$this->appid = $alipay_config["app_id"];
		$this->private_key = $alipay_config["merchant_private_key"];
		$this->alipay_public_key = $alipay_config["alipay_public_key"];
		$this->charset = $alipay_config["charset"];
		$this->MaxQueryRetry = $alipay_config["MaxQueryRetry"];
		$this->QueryDuration = $alipay_config["QueryDuration"];
		$this->notify_url = $alipay_config["notify_url"];
		if( empty($this->appid) || trim($this->appid) == "" ) 
		{
			throw new Exception("appid should not be NULL!");
		}
		if( empty($this->private_key) || trim($this->private_key) == "" ) 
		{
			throw new Exception("private_key should not be NULL!");
		}
		if( empty($this->alipay_public_key) || trim($this->alipay_public_key) == "" ) 
		{
			throw new Exception("alipay_public_key should not be NULL!");
		}
		if( empty($this->charset) || trim($this->charset) == "" ) 
		{
			throw new Exception("charset should not be NULL!");
		}
		if( empty($this->QueryDuration) || trim($this->QueryDuration) == "" ) 
		{
			throw new Exception("QueryDuration should not be NULL!");
		}
		if( empty($this->gateway_url) || trim($this->gateway_url) == "" ) 
		{
			throw new Exception("gateway_url should not be NULL!");
		}
		if( empty($this->MaxQueryRetry) || trim($this->MaxQueryRetry) == "" ) 
		{
			throw new Exception("MaxQueryRetry should not be NULL!");
		}
	}
	public function AlipayWapPayService($alipay_config) 
	{
		$this->__construct($alipay_config);
	}
	public function refund($req) 
	{
		$bizContent = $req->getBizContent();
		$this->writeLog($bizContent);
		$request = new AlipayTradeRefundRequest();
		$request->setBizContent($bizContent);
		$response = $this->aopclientRequestExecute($request, NULL, $req->getAppAuthToken());
		$response = $response->alipay_trade_refund_response;
		$result = new AlipayF2FRefundResult($response);
		if( !empty($response) && "10000" == $response->code ) 
		{
			$result->setTradeStatus("SUCCESS");
		}
		else 
		{
			if( $this->tradeError($response) ) 
			{
				$result->setTradeStatus("UNKNOWN");
			}
			else 
			{
				$result->setTradeStatus("FAILED");
			}
		}
		return $result;
	}
	public function qrPay($req) 
	{
		$bizContent = $req->getBizContent();
		$this->writeLog($bizContent);
		$request = new AlipayTradePrecreateRequest();
		$request->setBizContent($bizContent);
		$request->setNotifyUrl($this->notify_url);
		$response = $this->aopclientRequestExecute($request, NULL, $req->getAppAuthToken());
		$response = $response->alipay_trade_precreate_response;
		$result = new AlipayF2FPrecreateResult($response);
		if( !empty($response) && "10000" == $response->code ) 
		{
			$result->setTradeStatus("SUCCESS");
		}
		else 
		{
			if( $this->tradeError($response) ) 
			{
				$result->setTradeStatus("UNKNOWN");
			}
			else 
			{
				$result->setTradeStatus("FAILED");
			}
		}
		return $result;
	}
	public function queryTradeResult($req) 
	{
		$response = $this->query($req);
		$result = new AlipayF2FQueryResult($response);
		if( $this->querySuccess($response) ) 
		{
			$result->setTradeStatus("SUCCESS");
		}
		else 
		{
			if( $this->tradeError($response) ) 
			{
				$result->setTradeStatus("UNKNOWN");
			}
			else 
			{
				$result->setTradeStatus("FAILED");
			}
		}
		return $result;
	}
	public function check($arr) 
	{
		$aop = new AopClient();
		$aop->alipayrsaPublicKey = $this->alipay_public_key;
		$result = $aop->rsaCheckV1($arr, $this->alipay_public_key);
		return $result;
	}
	public function query($queryContentBuilder) 
	{
		$biz_content = $queryContentBuilder->getBizContent();
		$this->writeLog($biz_content);
		$request = new AlipayTradeQueryRequest();
		$request->setBizContent($biz_content);
		$response = $this->aopclientRequestExecute($request, NULL, $queryContentBuilder->getAppAuthToken());
		return $response->alipay_trade_query_response;
	}
	protected function loopQueryResult($queryContentBuilder) 
	{
		$queryResult = NULL;
		for( $i = 1; $i < $this->MaxQueryRetry; $i++ ) 
		{
			try 
			{
				sleep($this->QueryDuration);
			}
			catch( Exception $e ) 
			{
				$e->getMessage();
				print $e->getMessage();
				exit();
			}
			$queryResponse = $this->query($queryContentBuilder);
			if( !empty($queryResponse) ) 
			{
				if( $this->stopQuery($queryResponse) ) 
				{
					return $queryResponse;
				}
				$queryResult = $queryResponse;
			}
		}
		return $queryResult;
	}
	protected function stopQuery($response) 
	{
		if( "10000" == $response->code && ("TRADE_FINISHED" == $response->trade_status || "TRADE_SUCCESS" == $response->trade_status || "TRADE_CLOSED" == $response->trade_status) ) 
		{
			return true;
		}
		return false;
	}
	private function checkQueryAndCancel($outTradeNo, $appAuthToken, $result, $queryResponse) 
	{
		if( $this->querySuccess($queryResponse) ) 
		{
			$result->setTradeStatus("SUCCESS");
			$result->setResponse($queryResponse);
			return $result;
		}
		if( $this->queryClose($queryResponse) ) 
		{
			$result->setTradeStatus("FAILED");
			return $result;
		}
		$cancelContentBuilder = new AlipayTradeCancelContentBuilder();
		$cancelContentBuilder->setAppAuthToken($appAuthToken);
		$cancelContentBuilder->setOutTradeNo($outTradeNo);
		$cancelResponse = $this->cancel($cancelContentBuilder);
		if( $this->tradeError($cancelResponse) ) 
		{
			$result->setTradeStatus("UNKNOWN");
		}
		else 
		{
			$result->setTradeStatus("FAILED");
		}
		return $result;
	}
	protected function querySuccess($queryResponse) 
	{
		return !empty($queryResponse) && $queryResponse->code == "10000" && ($queryResponse->trade_status == "TRADE_SUCCESS" || $queryResponse->trade_status == "TRADE_FINISHED");
	}
	protected function queryClose($queryResponse) 
	{
		return !empty($queryResponse) && $queryResponse->code == "10000" && $queryResponse->trade_status == "TRADE_CLOSED";
	}
	protected function tradeError($response) 
	{
		return empty($response) || $response->code == "20000";
	}
	public function cancel($cancelContentBuilder) 
	{
		$biz_content = $cancelContentBuilder->getBizContent();
		$this->writeLog($biz_content);
		$request = new AlipayTradeCancelRequest();
		$request->setBizContent($biz_content);
		$response = $this->aopclientRequestExecute($request, NULL, $cancelContentBuilder->getAppAuthToken());
		return $response->alipay_trade_cancel_response;
	}
	private function aopclientRequestExecute($request, $token = NULL, $appAuthToken = NULL) 
	{
		$aop = new AopClient();
		$aop->gatewayUrl = $this->gateway_url;
		$aop->appId = $this->appid;
		$aop->rsaPrivateKey = $this->private_key;
		$aop->alipayrsaPublicKey = $this->alipay_public_key;
		$aop->apiVersion = "1.0";
		$aop->postCharset = $this->charset;
		$aop->format = $this->format;
		$aop->debugInfo = true;
		$result = $aop->execute($request, $token, $appAuthToken);
		return $result;
	}
	public function writeLog($text) 
	{
		file_put_contents("log/log.txt", date("Y-m-d H:i:s") . "  " . $text . "\r\n", FILE_APPEND);
	}
	public function create_erweima($content, $size = "500", $lev = "L", $margin = "0") 
	{
		$content = urlencode($content);
		$image = "<style>.qrcode-area{margin:0 auto;position:relative}\n.qrcode-integration .qrcode-header{display:block;width:auto;margin:0;padding:0;margin-top:75px;margin-bottom:16px}\n.qrcode-header-money{font-size:26px;font-weight:700;color:#f60}\n.qrcode-integration .qrcode-img-area{width:213px;height:215px;text-align:center}\n.qrcode-img-area.qrcode-img-crash{height:220px}\n.qrcode-reward-wrapper{text-align:center}\n.qrcode-reward{display:inline-block;margin:0;padding:2px 5px;background-color:#0188cd;border-radius:0;font-size:12px;line-height:16px;color:#fff}\n.qrcode-reward-question{font-size:12px;margin-left:5px;margin-right:0}\n.qrcode-integration .qrcode-loading{top:70px;left:60px}\n.qrcode-integration .qrcode-img{top:70px;left:70px}\n.qrcode-integration .qrcode-img-wrapper{position:relative;width:225px;height:auto;top:6px;min-height:168px;margin:0 auto;border:1px solid #d3d3d3;-webkit-box-shadow:1px 1px 1px #ccc;box-shadow:1px 1px 1px #ccc;background:#fff}\n.qrcode-img-area .qrcode-busy-icon{padding-top:15px}\n.qrcode-img-area .qrcode-busy-text{margin-top:20px}\na.mi-button-lwhite .mi-button-text{padding:8px 39px 4px 36px}\n.qrcode-img-area .mi-button{margin-top:40px}\n.qrcode-img-explain{padding:5px 0 50px}\n.qrcode-img-explain img{margin-left:37px;margin-top:5px}\n.qrcode-img-explain div{margin-left:10px}\n.fn-left{float:left}\n.qrcode-foot{text-align:center}\n.qrcode-downloadApp,.qrcode-downloadApp:active,.qrcode-downloadApp:hover,.qrcode-explain a.qrcode-downloadApp:hover{font-size:12px;color:#a6a6a6;text-decoration:underline}\n.area-split{margin-top:156px;width:10px;height:300px;background-image:url(https://t.alipayobjects.com/images/T1PspfXixsXXXXXXXX.png);background-repeat:no-repeat}\n.qrguide-area{position:absolute;top:62px;left:505px;width:204px;height:183px;cursor:pointer}\n.qrguide-area .qrguide-area-img{display:block;position:absolute;bottom:0;left:0;z-index:-1}\n.qrguide-area .qrguide-area-img.active{z-index:10}\n.qrguide-area .qrguide-area-img.background{z-index:9}\n.qrcode-notice .qrcode-notice-title{padding:10px 10px 11px 63px}\n</style><div class=\"qrcode-integration qrcode-area\"><div class=\"qrcode-img-wrapper\" data-role=\"qrPayImgWrapper\"><div data-role=\"qrPayImg\" class=\"qrcode-img-area\"><div style=\"position: relative;display: inline-block;\"><img src=\"//pan.baidu.com/share/qrcode?w=" . $size . "&h=" . $size . "&url=" . $content . "\" width=\"223\" height=\"223\" style=\"float: left;\"><img src=\"https://t.alipayobjects.com/tfscom/T1Z5XfXdxmXXXXXXXX.png\" style=\"position: absolute;top: 50%;left: 50%;width:42px;height:42px;margin-left: -21px;margin-top: -21px\"></div></div><div class=\"qrcode-img-explain fn-clear\"><img class=\"fn-left\" src=\"https://t.alipayobjects.com/images/T1bdtfXfdiXXXXXXXX.png\" alt=\"扫一扫标识\" seed=\"qrcodeImgExplain-tImagesT1bdtfXfdiXXXXXXXX\" smartracker=\"on\"><div class=\"fn-left\">打开手机支付宝<br>扫一扫继续付款</div></div></div></div>";
		return $image;
	}
}
class AlipayF2FPayResult 
{
	private $tradeStatus = NULL;
	private $response = NULL;
	public function __construct($response) 
	{
		$this->response = $response;
	}
	public function AlipayF2FPayResult($response) 
	{
		$this->__construct($response);
	}
	public function setTradeStatus($tradeStatus) 
	{
		$this->tradeStatus = $tradeStatus;
	}
	public function getTradeStatus() 
	{
		return $this->tradeStatus;
	}
	public function setResponse($response) 
	{
		$this->response = $response;
	}
	public function getResponse() 
	{
		return $this->response;
	}
}
class AlipayF2FPrecreateResult 
{
	private $tradeStatus = NULL;
	private $response = NULL;
	public function __construct($response) 
	{
		$this->response = $response;
	}
	public function AlipayF2FPrecreateResult($response) 
	{
		$this->__construct($response);
	}
	public function setTradeStatus($tradeStatus) 
	{
		$this->tradeStatus = $tradeStatus;
	}
	public function getTradeStatus() 
	{
		return $this->tradeStatus;
	}
	public function setResponse($response) 
	{
		$this->response = $response;
	}
	public function getResponse() 
	{
		return $this->response;
	}
}
class AlipayF2FQueryResult 
{
	private $tradeStatus = NULL;
	private $response = NULL;
	public function __construct($response) 
	{
		$this->response = $response;
	}
	public function AlipayF2FPayResult($response) 
	{
		$this->__construct($response);
	}
	public function setTradeStatus($tradeStatus) 
	{
		$this->tradeStatus = $tradeStatus;
	}
	public function getTradeStatus() 
	{
		return $this->tradeStatus;
	}
	public function setResponse($response) 
	{
		$this->response = $response;
	}
	public function getResponse() 
	{
		return $this->response;
	}
}
class AlipayF2FRefundResult 
{
	private $tradeStatus = NULL;
	private $response = NULL;
	public function __construct($response) 
	{
		$this->response = $response;
	}
	public function AlipayF2FPayResult($response) 
	{
		$this->__construct($response);
	}
	public function setTradeStatus($tradeStatus) 
	{
		$this->tradeStatus = $tradeStatus;
	}
	public function getTradeStatus() 
	{
		return $this->tradeStatus;
	}
	public function setResponse($response) 
	{
		$this->response = $response;
	}
	public function getResponse() 
	{
		return $this->response;
	}
}
class ContentBuilder 
{
	private $appAuthToken = NULL;
	private $notifyUrl = NULL;
	public function setAppAuthToken($appAuthToken) 
	{
		$this->appAuthToken = $appAuthToken;
	}
	public function setNotifyUrl($notifyUrl) 
	{
		$this->notifyUrl = $notifyUrl;
	}
	public function getAppAuthToken() 
	{
		return $this->appAuthToken;
	}
	public function getNotifyUrl() 
	{
		return $this->notifyUrl;
	}
}
class AlipayTradeQueryContentBuilder extends ContentBuilder 
{
	private $tradeNo = NULL;
	private $outTradeNo = NULL;
	private $bizContentarr = array( );
	private $bizContent = NULL;
	public function getBizContent() 
	{
		if( !empty($this->bizContentarr) ) 
		{
			$this->bizContent = json_encode($this->bizContentarr, JSON_UNESCAPED_UNICODE);
		}
		return $this->bizContent;
	}
	public function getOutTradeNo() 
	{
		return $this->outTradeNo;
	}
	public function setOutTradeNo($outTradeNo) 
	{
		$this->outTradeNo = $outTradeNo;
		$this->bizContentarr["out_trade_no"] = $outTradeNo;
	}
	public function getTradeNo() 
	{
		return $this->tradeNo;
	}
	public function setTradeNo($tradeNo) 
	{
		$this->tradeNo = $tradeNo;
		$this->bizContentarr["trade_no"] = $tradeNo;
	}
}
class GoodsDetail 
{
	private $goodsId = NULL;
	private $alipayGoodsId = NULL;
	private $goodsName = NULL;
	private $quantity = NULL;
	private $price = NULL;
	private $goodsCategory = NULL;
	private $body = NULL;
	private $goodsDetail = array( );
	public function getGoodsDetail() 
	{
		return $this->goodsDetail;
	}
	public function setGoodsId($goodsId) 
	{
		$this->goodsId = $goodsId;
		$this->goodsDetail["goods_id"] = $goodsId;
	}
	public function getGoodsId() 
	{
		return $this->goodsId;
	}
	public function setAlipayGoodsId($alipayGoodsId) 
	{
		$this->alipayGoodsId = $alipayGoodsId;
		$this->goodsDetail["alipay_goods_id"] = $alipayGoodsId;
	}
	public function getAlipayGoodsId() 
	{
		return $this->alipayGoodsId;
	}
	public function setGoodsName($goodsName) 
	{
		$this->goodsName = $goodsName;
		$this->goodsDetail["goods_name"] = $goodsName;
	}
	public function getGoodsName() 
	{
		return $this->goodsName;
	}
	public function setQuantity($quantity) 
	{
		$this->quantity = $quantity;
		$this->goodsDetail["quantity"] = $quantity;
	}
	public function getQuantity() 
	{
		return $this->quantity;
	}
	public function setPrice($price) 
	{
		$this->price = $price;
		$this->goodsDetail["price"] = $price;
	}
	public function getPrice() 
	{
		return $this->price;
	}
	public function setGoodsCategory($goodsCategory) 
	{
		$this->goodsCategory = $goodsCategory;
		$this->goodsDetail["goods_category"] = $goodsCategory;
	}
	public function getGoodsCategory() 
	{
		return $this->goodsCategory;
	}
	public function setBody($body) 
	{
		$this->body = $body;
		$this->goodsDetail["body"] = $body;
	}
	public function getBody() 
	{
		return $this->body;
	}
}
class ExtendParams 
{
	private $sysServiceProviderId = NULL;
	private $hbFqNum = NULL;
	private $hbFqSellerPercent = NULL;
	private $extendParamsArr = array( );
	public function getExtendParams() 
	{
		if( !empty($this->extendParamsArr) ) 
		{
			return $this->extendParamsArr;
		}
	}
	public function getSysServiceProviderId() 
	{
		return $this->sysServiceProviderId;
	}
	public function setSysServiceProviderId($sysServiceProviderId) 
	{
		$this->sysServiceProviderId = $sysServiceProviderId;
		$this->extendParamsArr["sys_service_provider_id"] = $sysServiceProviderId;
	}
	public function getHbFqNum() 
	{
		return $this->hbFqNum;
	}
	public function setHbFqNum($hbFqNum) 
	{
		$this->hbFqNum = $hbFqNum;
		$this->extendParamsArr["hb_fq_num"] = $hbFqNum;
	}
	public function getHbFqSellerPercent() 
	{
		return $this->hbFqSellerPercent;
	}
	public function setHbFqSellerPercent($hbFqSellerPercent) 
	{
		$this->hbFqSellerPercent = $hbFqSellerPercent;
		$this->extendParamsArr["hb_fq_seller_percent"] = $hbFqSellerPercent;
	}
}
class RoyaltyDetailInfo 
{
	private $serialNo = NULL;
	private $transInType = NULL;
	private $batchNo = NULL;
	private $outRelationId = NULL;
	private $transOutType = NULL;
	private $transOut = NULL;
	private $transIn = NULL;
	private $amount = NULL;
	private $desc = NULL;
	private $royaltyDetailInfo = array( );
	public function __construct() 
	{
		$this->setTransInType("userId");
		$this->setTransOutType("userId");
	}
	public function RoyaltyDetailInfo() 
	{
		$this->__construct();
	}
	public function getRoyaltyDetailInfo() 
	{
		return $this->royaltyDetailInfo;
	}
	public function getAmount() 
	{
		return $this->amount;
	}
	public function getBatchNo() 
	{
		return $this->batchNo;
	}
	public function getDesc() 
	{
		return $this->desc;
	}
	public function getOutRelationId() 
	{
		return $this->outRelationId;
	}
	public function getSerialNo() 
	{
		return $this->serialNo;
	}
	public function getTransIn() 
	{
		return $this->transIn;
	}
	public function getTransInType() 
	{
		return $this->transInType;
	}
	public function getTransOut() 
	{
		return $this->transOut;
	}
	public function getTransOutType() 
	{
		return $this->transOutType;
	}
	public function setAmount($amount) 
	{
		$this->amount = $amount;
		$this->royaltyDetailInfo["amount"] = $amount;
	}
	public function setBatchNo($batchNo) 
	{
		$this->batchNo = $batchNo;
		$this->royaltyDetailInfo["batch_no"] = $batchNo;
	}
	public function setDesc($desc) 
	{
		$this->desc = $desc;
		$this->royaltyDetailInfo["desc"] = $desc;
	}
	public function setOutRelationId($outRelationId) 
	{
		$this->outRelationId = $outRelationId;
		$this->royaltyDetailInfo["out_relation_id"] = $outRelationId;
	}
	public function setSerialNo($serialNo) 
	{
		$this->serialNo = $serialNo;
		$this->royaltyDetailInfo["serial_no"] = $serialNo;
	}
	public function setTransIn($transIn) 
	{
		$this->transIn = $transIn;
		$this->royaltyDetailInfo["trans_in"] = $transIn;
	}
	public function setTransInType($transInType) 
	{
		$this->transInType = $transInType;
		$this->royaltyDetailInfo["trans_in_type"] = $transInType;
	}
	public function setTransOut($transOut) 
	{
		$this->transOut = $transOut;
		$this->royaltyDetailInfo["trans_out"] = $transOut;
	}
	public function setTransOutType($transOutType) 
	{
		$this->transOutType = $transOutType;
		$this->royaltyDetailInfo["trans_out_type"] = $transOutType;
	}
}
class AlipayTradePrecreateContentBuilder extends ContentBuilder 
{
	private $outTradeNo = NULL;
	private $sellerId = NULL;
	private $totalAmount = NULL;
	private $discountableAmount = NULL;
	private $undiscountableAmount = NULL;
	private $buyerLogonId = NULL;
	private $subject = NULL;
	private $body = NULL;
	private $goodsDetailList = array( );
	private $operatorId = NULL;
	private $storeId = NULL;
	private $alipayStoreId = NULL;
	private $terminalId = NULL;
	private $extendParams = array( );
	private $timeExpress = NULL;
	private $bizContent = NULL;
	private $bizParas = array( );
	public function setOutTradeNo($outTradeNo) 
	{
		$this->outTradeNo = $outTradeNo;
		$this->bizParas["out_trade_no"] = $outTradeNo;
	}
	public function getOutTradeNo() 
	{
		return $this->outTradeNo;
	}
	public function setSellerId($sellerId) 
	{
		$this->sellerId = $sellerId;
		$this->bizParas["seller_id"] = $sellerId;
	}
	public function getSellerId() 
	{
		return $this->sellerId;
	}
	public function setTotalAmount($totalAmount) 
	{
		$this->totalAmount = $totalAmount;
		$this->bizParas["total_amount"] = $totalAmount;
	}
	public function getTotalAmount() 
	{
		return $this->totalAmount;
	}
	public function setDiscountableAmount($discountableAmount) 
	{
		$this->discountableAmount = $discountableAmount;
		$this->bizParas["discountable_amount"] = $discountableAmount;
	}
	public function getDiscountableAmount() 
	{
		return $this->discountableAmount;
	}
	public function setUndiscountableAmount($undiscountableAmount) 
	{
		$this->undiscountableAmount = $undiscountableAmount;
		$this->bizParas["undiscountable_amount"] = $undiscountableAmount;
	}
	public function getUndiscountableAmount() 
	{
		return $this->undiscountableAmount;
	}
	public function setBuyerLogonId($buyerLogonId) 
	{
		$this->buyerLogonId = $buyerLogonId;
		$this->bizParas["buyer_logon_id"] = $buyerLogonId;
	}
	public function getBuyerLogonId() 
	{
		return $this->buyerLogonId;
	}
	public function setSubject($subject) 
	{
		$this->subject = $subject;
		$this->bizParas["subject"] = $subject;
	}
	public function getSubject() 
	{
		return $this->subject;
	}
	public function setBody($body) 
	{
		$this->body = $body;
		$this->bizParas["body"] = $body;
	}
	public function getBody() 
	{
		return $this->body;
	}
	public function setOperatorId($operatorId) 
	{
		$this->operatorId = $operatorId;
		$this->bizParas["operator_id"] = $operatorId;
	}
	public function getOperatorId() 
	{
		return $this->operatorId;
	}
	public function setStoreId($storeId) 
	{
		$this->storeId = $storeId;
		$this->bizParas["store_id"] = $storeId;
	}
	public function getStoreId() 
	{
		return $this->storeId;
	}
	public function setTerminalId($terminalId) 
	{
		$this->terminalId = $terminalId;
		$this->bizParas["terminal_id"] = $terminalId;
	}
	public function getTerminalId() 
	{
		return $this->terminalId;
	}
	public function setTimeExpress($timeExpress) 
	{
		$this->timeExpress = $timeExpress;
		$this->bizParas["timeout_express"] = $timeExpress;
	}
	public function getTimeExpress() 
	{
		return $this->timeExpress;
	}
	public function getAlipayStoreId() 
	{
		return $this->alipayStoreId;
	}
	public function setAlipayStoreId($alipayStoreId) 
	{
		$this->alipayStoreId = $alipayStoreId;
		$this->bizParas["alipay_store_id"] = $alipayStoreId;
	}
	public function getExtendParams() 
	{
		return $this->extendParams;
	}
	public function setExtendParams($extendParams) 
	{
		$this->extendParams = $extendParams;
		$this->bizParas["extend_params"] = $extendParams;
	}
	public function getGoodsDetailList() 
	{
		return $this->goodsDetailList;
	}
	public function setGoodsDetailList($goodsDetailList) 
	{
		$this->goodsDetailList = $goodsDetailList;
		$this->bizParas["goods_detail"] = $goodsDetailList;
	}
	public function getBizContent() 
	{
		if( !empty($this->bizParas) ) 
		{
			$this->bizContent = json_encode($this->bizParas, JSON_UNESCAPED_UNICODE);
		}
		return $this->bizContent;
	}
}
class AlipayTradePrecreateRequest 
{
	private $bizContent = NULL;
	private $apiParas = array( );
	private $terminalType = NULL;
	private $terminalInfo = NULL;
	private $prodCode = NULL;
	private $apiVersion = "1.0";
	private $notifyUrl = NULL;
	private $returnUrl = NULL;
	private $needEncrypt = false;
	public function setBizContent($bizContent) 
	{
		$this->bizContent = $bizContent;
		$this->apiParas["biz_content"] = $bizContent;
	}
	public function getBizContent() 
	{
		return $this->bizContent;
	}
	public function getApiMethodName() 
	{
		return "alipay.trade.precreate";
	}
	public function setNotifyUrl($notifyUrl) 
	{
		$this->notifyUrl = $notifyUrl;
	}
	public function getNotifyUrl() 
	{
		return $this->notifyUrl;
	}
	public function setReturnUrl($returnUrl) 
	{
		$this->returnUrl = $returnUrl;
	}
	public function getReturnUrl() 
	{
		return $this->returnUrl;
	}
	public function getApiParas() 
	{
		return $this->apiParas;
	}
	public function getTerminalType() 
	{
		return $this->terminalType;
	}
	public function setTerminalType($terminalType) 
	{
		$this->terminalType = $terminalType;
	}
	public function getTerminalInfo() 
	{
		return $this->terminalInfo;
	}
	public function setTerminalInfo($terminalInfo) 
	{
		$this->terminalInfo = $terminalInfo;
	}
	public function getProdCode() 
	{
		return $this->prodCode;
	}
	public function setProdCode($prodCode) 
	{
		$this->prodCode = $prodCode;
	}
	public function setApiVersion($apiVersion) 
	{
		$this->apiVersion = $apiVersion;
	}
	public function getApiVersion() 
	{
		return $this->apiVersion;
	}
	public function setNeedEncrypt($needEncrypt) 
	{
		$this->needEncrypt = $needEncrypt;
	}
	public function getNeedEncrypt() 
	{
		return $this->needEncrypt;
	}
}
class SignData 
{
	public $signSourceData = NULL;
	public $sign = NULL;
}
class AlipayTradeQueryRequest 
{
	private $bizContent = NULL;
	private $apiParas = array( );
	private $terminalType = NULL;
	private $terminalInfo = NULL;
	private $prodCode = NULL;
	private $apiVersion = "1.0";
	private $notifyUrl = NULL;
	private $returnUrl = NULL;
	private $needEncrypt = false;
	public function setBizContent($bizContent) 
	{
		$this->bizContent = $bizContent;
		$this->apiParas["biz_content"] = $bizContent;
	}
	public function getBizContent() 
	{
		return $this->bizContent;
	}
	public function getApiMethodName() 
	{
		return "alipay.trade.query";
	}
	public function setNotifyUrl($notifyUrl) 
	{
		$this->notifyUrl = $notifyUrl;
	}
	public function getNotifyUrl() 
	{
		return $this->notifyUrl;
	}
	public function setReturnUrl($returnUrl) 
	{
		$this->returnUrl = $returnUrl;
	}
	public function getReturnUrl() 
	{
		return $this->returnUrl;
	}
	public function getApiParas() 
	{
		return $this->apiParas;
	}
	public function getTerminalType() 
	{
		return $this->terminalType;
	}
	public function setTerminalType($terminalType) 
	{
		$this->terminalType = $terminalType;
	}
	public function getTerminalInfo() 
	{
		return $this->terminalInfo;
	}
	public function setTerminalInfo($terminalInfo) 
	{
		$this->terminalInfo = $terminalInfo;
	}
	public function getProdCode() 
	{
		return $this->prodCode;
	}
	public function setProdCode($prodCode) 
	{
		$this->prodCode = $prodCode;
	}
	public function setApiVersion($apiVersion) 
	{
		$this->apiVersion = $apiVersion;
	}
	public function getApiVersion() 
	{
		return $this->apiVersion;
	}
	public function setNeedEncrypt($needEncrypt) 
	{
		$this->needEncrypt = $needEncrypt;
	}
	public function getNeedEncrypt() 
	{
		return $this->needEncrypt;
	}
}
class SysCrypt 
{
	private $crypt_key = NULL;
	public function __construct($crypt_key) 
	{
		$this->crypt_key = $crypt_key;
	}
	public function php_encrypt($txt) 
	{
		srand((double) microtime() * 1000000);
		$encrypt_key = md5(rand(0, 32000));
		$ctr = 0;
		$tmp = "";
		for( $i = 0; $i < strlen($txt);
		$i++ ) 
		{
			$ctr = ($ctr == strlen($encrypt_key) ? 0 : $ctr);
			$tmp .= $encrypt_key[$ctr] . ($txt[$i] ^ $encrypt_key[$ctr++]);
		}
		return base64_encode(self::__key($tmp, $this->crypt_key));
	}
	public function php_decrypt($txt) 
	{
		$txt = self::__key(base64_decode($txt), $this->crypt_key);
		$tmp = "";
		for( $i = 0; $i < strlen($txt);
		$i++ ) 
		{
			$md5 = $txt[$i];
			$tmp .= $txt[++$i] ^ $md5;
		}
		return $tmp;
	}
	private function __key($txt, $encrypt_key) 
	{
		$encrypt_key = md5($encrypt_key);
		$ctr = 0;
		$tmp = "";
		for( $i = 0; $i < strlen($txt);
		$i++ ) 
		{
			$ctr = ($ctr == strlen($encrypt_key) ? 0 : $ctr);
			$tmp .= $txt[$i] ^ $encrypt_key[$ctr++];
		}
		return $tmp;
	}
	public function __destruct() 
	{
		$this->crypt_key = NULL;
	}
}
function alipay_encrypt($str, $screct_key) 
{
	$screct_key = base64_decode($screct_key);
	$str = trim($str);
	$str = addPKCS7Padding($str);
	$iv = mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_128, MCRYPT_MODE_CBC), 1);
	$encrypt_str = mcrypt_encrypt(MCRYPT_RIJNDAEL_128, $screct_key, $str, MCRYPT_MODE_CBC);
	return base64_encode($encrypt_str);
}
function alipay_decrypt($str, $screct_key) 
{
	$str = base64_decode($str);
	$screct_key = base64_decode($screct_key);
	$iv = mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_128, MCRYPT_MODE_CBC), 1);
	$encrypt_str = mcrypt_decrypt(MCRYPT_RIJNDAEL_128, $screct_key, $str, MCRYPT_MODE_CBC);
	$encrypt_str = trim($encrypt_str);
	$encrypt_str = stripPKSC7Padding($encrypt_str);
	return $encrypt_str;
}
function addPKCS7Padding($source) 
{
	$source = trim($source);
	$block = mcrypt_get_block_size(MCRYPT_RIJNDAEL_128, MCRYPT_MODE_CBC);
	$pad = $block - strlen($source) % $block;
	if( $pad <= $block ) 
	{
		$char = chr($pad);
		$source .= str_repeat($char, $pad);
	}
	return $source;
}
function stripPKSC7Padding($source) 
{
	$source = trim($source);
	$char = substr($source, -1);
	$num = ord($char);
	if( $num == 62 ) 
	{
		return $source;
	}
	$source = substr($source, 0, 0 - $num);
	return $source;
}
